@@ -172,6 +172,8 @@ The new version of the list, minus the deleted strings, is returned."
 	     (nreverse list))
 	(and work-buffer (kill-buffer work-buffer)))))))
 
+(defvar buffer-file-type)
+
 (defun vm-write-string (where string)
   (if (bufferp where)
       (vm-save-buffer-excursion
@@ -186,31 +188,17 @@ The new version of the list, minus the deleted strings, is returned."
 	  (save-excursion
 	    (setq temp-buffer (generate-new-buffer "*vm-work*"))
 	    (set-buffer temp-buffer)
+	    (setq selective-display nil)
 	    (insert string)
 	    ;; correct for VM's uses of this function---
 	    ;; writing out message separators
 	    (setq buffer-file-type nil)
-	    ;; Tell XEmacs/MULE to pick the correct newline conversion.
-	    (and vm-xemacs-mule-p
-		 (set-file-coding-system 'no-conversion nil))
+	    ;; Tell MULE to pick the correct newline conversion.
+	    (if (or vm-xemacs-mule-p vm-fsfemacs-mule-p)
+		(set-buffer-file-coding-system 'no-conversion nil))
 	    (write-region (point-min) (point-max) where t 'quiet))
 	(and temp-buffer (kill-buffer temp-buffer))))))
 
-(defmacro vm-marker (pos &optional buffer)
-  (list 'set-marker '(make-marker) pos buffer))
-
-(defmacro vm-increment (variable)
-  (list 'setq variable (list '1+ variable)))
-
-(defmacro vm-decrement (variable)
-  (list 'setq variable (list '1- variable)))
-
-(defmacro vm-select-folder-buffer ()
-  '(and vm-mail-buffer
-	(or (buffer-name vm-mail-buffer)
-	    (error "Folder buffer has been killed."))
-	(set-buffer vm-mail-buffer)))
-
 (defun vm-check-for-killed-summary ()
   (and (bufferp vm-summary-buffer) (null (buffer-name vm-summary-buffer))
        (let ((mp vm-message-list))
@@ -231,59 +219,11 @@ The new version of the list, minus the deleted strings, is returned."
   (and (bufferp vm-mail-buffer) (null (buffer-name vm-mail-buffer))
        (setq vm-mail-buffer nil)))
 
-(defmacro vm-error-if-folder-read-only ()
-  '(while vm-folder-read-only
-     (signal 'folder-read-only (list (current-buffer)))))
-
 (put 'folder-read-only 'error-conditions '(folder-read-only error))
 (put 'folder-read-only 'error-message "Folder is read-only")
 
-(defmacro vm-error-if-virtual-folder ()
-  '(and (eq major-mode 'vm-virtual-mode)
-	(error "%s cannot be applied to virtual folders." this-command)))
-
-(defmacro vm-build-threads-if-unbuilt ()
-  '(if (null vm-thread-obarray)
-       (vm-build-threads nil)))
-
 (defun vm-abs (n) (if (< n 0) (- n) n))
 
-;; save-restriction flubs restoring the clipping region if you
-;; (widen) and modify text outside the old region.
-;; This should do it right.
-(defmacro vm-save-restriction (&rest forms)
-  (let ((vm-sr-clip (make-symbol "vm-sr-clip"))
-	(vm-sr-min (make-symbol "vm-sr-min"))
-	(vm-sr-max (make-symbol "vm-sr-max")))
-    (list 'let (list (list vm-sr-clip '(> (buffer-size)
-					  (- (point-max) (point-min))))
-		     ;; this shouldn't be necessary but the
-		     ;; byte-compiler turns these into interned symbols
-		     ;; which utterly defeats the purpose of the
-		     ;; make-symbol calls above.  Soooo, until the compiler
-		     ;; is fixed, these must be made into (let ...)
-		     ;; temporaries so that nested calls to this macros
-		     ;; won't misbehave.
-		     vm-sr-min vm-sr-max)
-	  (list 'and vm-sr-clip
-		(list 'setq vm-sr-min '(set-marker (make-marker) (point-min)))
-		(list 'setq vm-sr-max '(set-marker (make-marker) (point-max))))
-	  (list 'unwind-protect (cons 'progn forms)
-		'(widen)
-		(list 'and vm-sr-clip
-		      (list 'progn
-			    (list 'narrow-to-region vm-sr-min vm-sr-max)
-			    (list 'set-marker vm-sr-min nil)
-			    (list 'set-marker vm-sr-max nil)))))))
-
-(defmacro vm-save-buffer-excursion (&rest forms)
-  (list 'let '((vm-sbe-buffer (current-buffer)))
-	(list 'unwind-protect
-	      (cons 'progn forms)
-	      '(and (not (eq vm-sbe-buffer (current-buffer)))
-		    (buffer-name vm-sbe-buffer)
-		    (set-buffer vm-sbe-buffer)))))
-
 (defun vm-last (list) (while (cdr-safe list) (setq list (cdr list))) list)
 
 (defun vm-vector-to-list (vector)
@@ -436,44 +376,6 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 	((markerp object) (copy-marker object))
 	(t object)))
 
-(defun vm-multiple-frames-possible-p () 
-  (cond (vm-xemacs-p 
-	 (or (memq 'win (device-matching-specifier-tag-list))
-	     (featurep 'tty-frames)))
-        (vm-fsfemacs-p 
-         (fboundp 'make-frame))))
- 
-(defun vm-mouse-support-possible-p () 
-  (cond (vm-xemacs-p 
-         (featurep 'window-system)) 
-        (vm-fsfemacs-p 
-         (fboundp 'track-mouse))))
- 
-(defun vm-mouse-support-possible-here-p ()
-  (cond (vm-xemacs-p
-	 (memq 'win (device-matching-specifier-tag-list)))
-	(vm-fsfemacs-p
-	 (memq window-system '(x win32)))))
-
-(defun vm-menu-support-possible-p ()
-  (cond (vm-xemacs-p
-	 (featurep 'menubar))
-	(vm-fsfemacs-p
-	 (fboundp 'menu-bar-mode))))
- 
-(defun vm-toolbar-support-possible-p ()
-  (and vm-xemacs-p (featurep 'toolbar)))
-
-(defun vm-multiple-fonts-possible-p ()
-  (cond (vm-xemacs-p
-	 (memq (device-type) '(x mswindows)))
-	(vm-fsfemacs-p
-	 (or (eq window-system 'x)
-	     (eq window-system 'win32)))))
-
-(defun vm-images-possible-here-p ()
-  (and vm-xemacs-p (memq (device-type) '(x mswindows))))
-
 (defun vm-run-message-hook (message &optional hook-variable)
   (save-excursion
     (set-buffer (vm-buffer-of message))
@@ -751,13 +653,6 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 	(setq list (cdr list))))
     list))
 
-(defmacro vm-assert (expression)
-  (list 'or expression
-	(list 'progn
-	      (list 'setq 'debug-on-error t)
-	      (list 'error "assertion failed: %S"
-		    (list 'quote expression)))))
-
 (defun vm-time-difference (t1 t2)
   (let (usecs secs 65536-secs carry)
     (setq usecs (- (nth 2 t1) (nth 2 t2)))
@@ -778,3 +673,50 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 (if (fboundp 'char-to-int)
     (fset 'vm-char-to-int 'char-to-int)
   (fset 'vm-char-to-int 'identity))
+
+(cond ((fboundp 'charsets-in-region)
+       (fset 'vm-charsets-in-region 'charsets-in-region))
+      ((fboundp 'find-charset-region)
+       (fset 'vm-charsets-in-region 'find-charset-region)))
+
+(cond ((fboundp 'coding-system-name)
+       (fset 'vm-coding-system-name 'coding-system-name))
+      (t
+       (fset 'vm-coding-system-name 'symbol-name)))
+
+(defun vm-collapse-whitespace ()
+  (goto-char (point-min))
+  (while (re-search-forward "[ \t\n]+" nil 0)
+    (replace-match " " t t)))
+
+(defun vm-fill-paragraphs-containing-long-lines (len start end)
+  (let ((done nil)
+	(buffer-read-only nil)
+	(fill-column vm-paragraph-fill-column)
+	;; user doesn't want long line, so set this to zero for them.
+	(filladapt-fill-column-forward-fuzz 0))
+    (save-excursion
+      (vm-save-restriction
+       (widen)
+       (or (markerp end) (setq end (vm-marker end)))
+       (goto-char start)
+       (while (not done)
+	 (re-search-forward "$" end t)
+	 (if (>= (current-column) len)
+	     (fill-paragraph nil))
+	 (forward-line)
+	 (setq done (>= (point) end)))))))
+
+(defun vm-make-message-id ()
+  (let (hostname
+	(time (current-time)))
+    (setq hostname (cond ((string-match "\\." (system-name))
+			  (system-name))
+			 ((and (stringp mail-host-address)
+			       (string-match "\\." mail-host-address))
+			  mail-host-address)
+			 (t "gargle.gargle.HOWL")))
+    (format "<%d.%d.%d.%d@%s>"
+	    (car time) (nth 1 time) (nth 2 time)
+	    (random 1000000)
+	    hostname)))
